import pilas

# Colores principales.
negro = pilas.motor.Color(0, 0, 0)
blanco = pilas.motor.Color(255, 255, 255)
rojo = pilas.motor.Color(255, 0, 0)
verde = pilas.motor.Color(0, 255, 0)
azul = pilas.motor.Color(0, 0, 255)
gris = pilas.motor.Color(128, 128, 128)

# Colores secundarios
amarillo = pilas.motor.Color(255, 255, 0)
magenta = pilas.motor.Color(255, 0, 255)
cyan = pilas.motor.Color(0, 255, 255)
grisclaro = pilas.motor.Color(192, 192, 192)
grisoscuro = pilas.motor.Color(100, 100, 100)
verdeoscuro = pilas.motor.Color(0, 128, 0)
azuloscuro = pilas.motor.Color(0, 0, 128)
naranja = pilas.motor.Color(255, 200, 0)
rosa = pilas.motor.Color(255, 175, 175)
violeta = pilas.motor.Color(128, 0, 255)
marron = pilas.motor.Color(153, 102, 0)



# Colores transparentes
negro = pilas.motor.Color(0, 0, 0, 160)
blanco = pilas.motor.Color(255, 255, 255, 160)
rojo_transparente = pilas.motor.Color(255, 0, 0, 160)
verde_transparente = pilas.motor.Color(0, 255, 0, 160)
azul_transparente = pilas.motor.Color(0, 0, 255, 160)
gris_transparente = pilas.motor.Color(128, 128, 128, 160)
